################################################################################
# CONFIG
################################################################################
$HostIP = $ENV:ComputerName
$HostURL = "http://$HostIP"
$SolutionFolder = "C:\Release\"
$SolutionName = "SharePointStu.SharePoint.ChartWebPart.wsp"
$AdminServiceName = "SPAdminV4"
$IsAdminServiceWasRunning = $true
Try
{
	################################################################################
	# 0. Enviromental Settings
	################################################################################
	$User = [Security.Principal.WindowsIdentity]::GetCurrent().Name
	$tmp = Get-Location
	Write-Host "Current User = " $User
	Write-Host "Current Location = " $tmp
	Write-Host "Server = " $HostURL

	################################################################################
	# 1. Add Sharepoint modules to PowerShell
	################################################################################
	Write-Host "Adding Sharepoint module to PowerShell" -NoNewline
	Add-PSSnapin Microsoft.Sharepoint.Powershell -ErrorAction Stop
	Write-Host " - Done."

	################################################################################
	# 2. Start SP Admin Services if it is shutdown
	################################################################################
	if ($(Get-Service $AdminServiceName).Status -eq "Stopped")
	{
		Write-Host "Starting SP Admin Services" -NoNewline
		$IsAdminServiceWasRunning = $false
		Start-Service $AdminServiceName -ErrorAction SilentlyContinue
		Write-Host " - Done."
	}

	################################################################################
	# 3. Retract and remove the solution and features
	################################################################################
    function Disable-SiteCollectionFeatures
	{
		[Array] $ReverseFeaturesArray = "SharePointStu.SharePoint.ChartWebPart_SharePointStu.ChartWebPart", "SharePointStu.SharePoint.Visifire_SharePointStu.Visifire"
		
		foreach($feature in $ReverseFeaturesArray)
		{
			Write-Host "Disable feature $feature" -NoNewline
			Disable-SPFeature Identity $feature -Url $HostURL -Confirm:$false -Force -ErrorAction SilentlyContinue			
			Write-Host " - Done."
		}
	}
    
    function Restart-Services
    {
        Write-Host "Restarting Services: W3SVC,IISADMIN,SPAdminV4" -NoNewline
        [Array] $serviceArray = "W3SVC", "IISADMIN", "SPAdminV4"
        Restart-Service -Name $serviceArray
        Write-Host " - Done."
    }
	
	function Retract-Solution
	{
		Write-Host "Retracting solution: $SolutionName" -NoNewline
		$Solution = Get-SPSolution | ? {($_.Name -eq $SolutionName) -and ($_.Deployed -eq $true)}
		if ($Solution -ne $null)
		{
			if($Solution.ContainsWebApplicationResource)
			{
				Uninstall-SPSolution $SolutionName -AllWebApplications -Confirm:$false -ErrorAction SilentlyContinue
			}
			else
			{
				Uninstall-SPSolution $SolutionName -Confirm:$false -ErrorAction SilentlyContinue
			}
		}    
		while ($Solution.JobExists)
		{
			Start-Sleep 2
		}
		Write-Host " - Done."
	}

	function Remove-Solution
	{
		Write-Host "Removing solution: $SolutionName" -NoNewline
		if ($(Get-SPSolution | ? {$_.Name -eq $SolutionName}).Deployed -eq $false)
		{
			Remove-SPSolution $SolutionName -Confirm:$false -ErrorAction SilentlyContinue
		} 
		Write-Host " - Done."
	}
	
    Disable-SiteCollectionFeatures
    Restart-Services
	Retract-Solution	
	Remove-Solution

	################################################################################
	# 4. Add and deploy the solution
	#	 Enable site collection features
	################################################################################
	function Add-Solution
	{
		Write-Host "Adding solution: $SolutionName" -NoNewline
		$SolutionPath = $SolutionFolder + $SolutionName
		Add-SPSolution $SolutionPath | Out-Null -ErrorAction Stop
		Write-Host " - Done."
	}

	function Deploy-Solution
	{
		Write-Host "Deploying solution: $SolutionName" -NoNewline
		$Solution = Get-SPSolution | ? {($_.Name -eq $SolutionName) -and ($_.Deployed -eq $false)}
		if(($Solution -ne $null) -and ($Solution.ContainsWebApplicationResource))
		{
			Install-SPSolution $SolutionName -AllWebApplications -GACDeployment -Confirm:$false -Force -ErrorAction Stop
		}
		else
		{
			Install-SPSolution $SolutionName -GACDeployment -Confirm:$false -Force -ErrorAction Stop
		}
		while ($Solution.Deployed -eq $false)
		{
			Start-Sleep 2
		}
		Write-Host " - Done."
	}

	Add-Solution
	Deploy-Solution	
	
	################################################################################
	# 5. Stop SP Admin Services if it was previously shutdown
	################################################################################
	if (-not $IsAdminServiceWasRunning)
	{
		Write-Host "Stopping SP Admin Services" -NoNewline
		Stop-Service $AdminServiceName -ErrorAction SilentlyContinue
		Write-Host " - Done."
	}
}
Catch
{
	Write-Host "Error in Deployment: " $Error[0]  
	throw
	Exit 1
}	
Finally
{
	################################################################################
	# 6. Remove Sharepoint modules to PowerShell
	################################################################################
	Write-Host "Removing Sharepoint module to PowerShell" -NoNewline
	Remove-PSSnapin Microsoft.Sharepoint.Powershell -ErrorAction SilentlyContinue
	Write-Host " - Done."
}
Echo Finished